from enum import Enum


class Vulnerability:
    def __init__(self, kind=None, description=None, transactions=None):
        self.type = kind
        self.description = description
        self.transactions = transactions
        self.tested = False
        self.confirmed = False

    def __str__(self):
        return """
        Type = {type}
        Description = {description}
        Transactions = {transactions}
        """.format(
            type=self.type, description=self.description, transactions=self.transactions
        )


class VulnerabilityType(Enum):
    KILL_ONLY = 0
    KILL_AND_WITHDRAW = 1
    ETHER_THEFT = 2
